# -*- coding: utf-8 -*-

import shutil

from django.db import models
from django.conf import settings
from django.db.models import signals

from utils.constantes import TIPO_PERCENTUAL
from utils.constantes import PERCENTUAL_MINIMO

class Ano(models.Model):
    ano = models.CharField(max_length=4)

    class Meta:
        verbose_name = 'ano'
        verbose_name_plural = 'anos'

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.ano.capitalize()
        else:
            return self.ano


class Mes(models.Model):
    nome = models.CharField(max_length=20)
    codigo = models.CharField(unique=True, max_length=2)

    class Meta:
        verbose_name = 'mês'
        verbose_name_plural = 'meses'
        ordering = ['codigo']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class Estado(models.Model):
    nome = models.CharField(max_length=100)

    class Meta:
        ordering = ['nome']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class Municipio(models.Model):
    nome = models.CharField(max_length=100)
    estado = models.ForeignKey(Estado)

    class Meta:
        verbose_name = 'município'
        verbose_name_plural = 'municípios'
        ordering = ['nome']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class TipoRelatorio(models.Model):
    nome = models.CharField(max_length=100)

    class Meta:
        verbose_name = 'tipo de relatório'
        verbose_name_plural = 'tipos de relatório'
        ordering = ['nome']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class TipoCliente(models.Model):
    nome = models.CharField(max_length=100, unique=True)

    class Meta:
        verbose_name = 'tipo de cliente'
        verbose_name_plural = 'tipos de cliente'
        ordering = ['nome']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class Cliente(models.Model):
    """ Clientes que pertence a um determinado município.

    FUNDEB, Assistência social, Saúde, etc...
    """
    nome = models.CharField(max_length=200, unique=True)
    tipo_cliente = models.ForeignKey(TipoCliente)
    municipio = models.ForeignKey(Municipio)
    ativo = models.BooleanField(default=True)

    class Meta:
        ordering = ['municipio']

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class Indice(models.Model):
    nome = models.CharField(max_length=50, unique=True)
    somente_cliente = models.BooleanField(
        default=False,
        help_text='Este índice só pode ser usado na configuração de cliente, '+
        'não na configuração de tipo de cliente.'
    )

    class Meta:
        verbose_name = 'índice'
        verbose_name_plural = 'índices'

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.nome.capitalize()
        else:
            return self.nome


class IndiceClienteManager(models.Manager):
    def get_query_set(self):
        qs = super(IndiceClienteManager, self).get_query_set()
        qs = qs.filter(somente_cliente = True)
        return qs


class IndiceCliente(Indice):
    class Meta:
        proxy = True
        verbose_name = 'Índice para clientes'
        verbose_name_plural = 'Índices para clientes'
        ordering = ['nome']

    objects = IndiceClienteManager()

    def save(self, *args, **kwargs):
        if not self.id:
            self.somente_cliente = True
        return super(IndiceCliente, self).save(*args, **kwargs)


class IndiceTipoClienteManager(models.Manager):
    def get_query_set(self):
        qs = super(IndiceTipoClienteManager, self).get_query_set()
        qs = qs.filter(somente_cliente = False)
        return qs


class IndiceTipoCliente(Indice):
    class Meta:
        proxy = True
        verbose_name = 'Índice para tipos de cliente'
        verbose_name_plural = 'Índices para tipos de cliente'
        ordering = ['nome']

    objects = IndiceTipoClienteManager()

    def save(self, *args, **kwargs):
        if not self.id:
            self.somente_cliente = False
        return super(IndiceTipoCliente, self).save(*args, **kwargs)


class IndiceMetaBase(models.Model):
    ano = models.ForeignKey(Ano)
    tipo_percentual = models.IntegerField(
        choices=TIPO_PERCENTUAL,
        default=PERCENTUAL_MINIMO,
        verbose_name="tipo de meta"
    )
    valor = models.FloatField()


def my_custom_sql(sql):
    from django.db import connection, transaction
    cursor = connection.cursor()

    # Data modifying operation - commit required
    cursor.execute(sql)
    transaction.commit_unless_managed()


def cliente_pre_save(signal, instance, sender, **kwargs):
    # Verifica se está sendo realizado um update
    if instance.id:
        protocolos_arquivo = instance.protocoloarquivo_set.all()
        path_completo_antigo = None

        for protocolo_arquivo in protocolos_arquivo:
            for arquivo in protocolo_arquivo.arquivo_set.all():
                if not path_completo_antigo:
                    path_completo_antigo = arquivo.arquivo.path

                path_relativo_antigo = arquivo.arquivo.name
                novo_dst = path_relativo_antigo.rsplit('/', 4)[0] + '/' +\
                instance.nome + '/' +\
                path_relativo_antigo.split('/', 4)[-1]

                sql = 'update ' + arquivo._meta.db_table +\
                ' set ' + arquivo.arquivo.field.column + '=\'' + novo_dst + '\' where id=' + unicode(arquivo.id)

                my_custom_sql(sql.lower())

        if path_completo_antigo:
            src = path_completo_antigo.rsplit('/', 3)[0]
            dst = path_completo_antigo.rsplit('/', 4)[0] + '/' + instance.nome

            shutil.move(src.lower(), dst.lower())

def municipio_pre_save(signal, instance, sender, **kwargs):
    # Verifica se está sendo realizado um update
    if instance.id:
        protocolos_arquivo = instance.protocoloarquivo_set.all()
        path_completo_antigo = None

        for protocolo_arquivo in protocolos_arquivo:
            for arquivo in protocolo_arquivo.arquivo_set.all():
                if not path_completo_antigo:
                    path_completo_antigo = arquivo.arquivo.path

                path_relativo_antigo = arquivo.arquivo.name
                novo_dst = path_relativo_antigo.rsplit('/', 5)[0] + '/' +\
                instance.nome + '/' +\
                path_relativo_antigo.split('/', 3)[-1]

                sql = 'update ' + arquivo._meta.db_table +\
                ' set ' + arquivo.arquivo.field.column + '=\'' +\
                novo_dst + '\' where id=' + unicode(arquivo.id)

                my_custom_sql(sql.lower())

        if path_completo_antigo:
            src = path_completo_antigo.rsplit('/', 4)[0]
            dst = path_completo_antigo.rsplit('/', 5)[0] + '/' + instance.nome

            shutil.move(src.lower(), dst.lower())

def estado_pre_save(signal, instance, sender, **kwargs):
    # Verifica se está sendo realizado um update
    if instance.id:
        for municipio in instance.municipio_set.all():
            protocolos_arquivo = municipio.protocoloarquivo_set.all()
            path_completo_antigo = None

            for protocolo_arquivo in protocolos_arquivo:
                for arquivo in protocolo_arquivo.arquivo_set.all():
                    if not path_completo_antigo:
                        path_completo_antigo = arquivo.arquivo.path

                    path_relativo_antigo = arquivo.arquivo.name
                    novo_dst = path_relativo_antigo.rsplit('/', 8)[0] + '/' +\
                    instance.nome + '/' +\
                    path_relativo_antigo.split('/', 2)[-1]

                    sql = 'update ' + arquivo._meta.db_table +\
                    ' set ' + arquivo.arquivo.field.column + '=\'' +\
                    novo_dst + '\' where id=' + unicode(arquivo.id)

                    my_custom_sql(sql.lower())
            
            if path_completo_antigo:
                src = path_completo_antigo.rsplit('/', 5)[0]
                dst = path_completo_antigo.rsplit('/', 6)[0] + '/' + instance.nome

                shutil.move(src.lower(), dst.lower())

def mes_pre_save(signal, instance, sender, **kwargs):
    # Verifica se está sendo realizado um update
    if instance.id:
        protocolos_arquivo = instance.protocoloarquivo_set.all()
        path_completo_antigo = None

        if len(unicode(instance.codigo)) == 1:
            mes = '0' + unicode(instance.codigo)
        else:
            mes = unicode(instance.codigo)

        mes = mes + '_' + instance.nome

        for protocolo_arquivo in protocolos_arquivo:
            for arquivo in protocolo_arquivo.arquivo_set.all():
                path_completo_antigo = arquivo.arquivo.path

                path_relativo_antigo = arquivo.arquivo.name

                novo_dst = path_relativo_antigo.rsplit('/', 2)[0] + '/' +\
                mes + '/' +\
                path_relativo_antigo.rsplit('/', 1)[-1]

                sql = 'update ' + arquivo._meta.db_table +\
                ' set ' + arquivo.arquivo.field.column + '=\'' +\
                novo_dst + '\' where id=' + unicode(arquivo.id)

                my_custom_sql(sql.lower())

            src = path_completo_antigo.rsplit('/', 1)[0]
            dst = path_completo_antigo.rsplit('/', 2)[0] + '/' + mes

            shutil.move(src.lower(), dst.lower())

signals.pre_save.connect(mes_pre_save, sender=Mes, dispatch_uid='mes_pre_save')
signals.pre_save.connect(estado_pre_save, sender=Estado, dispatch_uid='estado_pre_save')
signals.pre_save.connect(cliente_pre_save, sender=Cliente, dispatch_uid='cliente_pre_save')
signals.pre_save.connect(municipio_pre_save, sender=Municipio, dispatch_uid='municipio_pre_save')
