# -*- coding: utf-8 -*-

import re
import datetime
import shutil
import os.path
from unicodedata import normalize

from django.template.defaultfilters import slugify
from django.db import models
from django.db.models import signals
from django.conf import settings
from django.contrib.auth.models import User

from cadastro.models import Cliente
from cadastro.models import Municipio
from cadastro.models import TipoRelatorio
from cadastro.models import IndiceCliente
from cadastro.models import IndiceTipoCliente
from cadastro.models import Ano
from cadastro.models import Mes
from utils.utils import formata_dia_mes
from utils.utils import get_ano_atual 
from utils.utils import get_mes_atual 
from utils.utils import get_dia_atual 
from utils.utils import get_hora_atual 
from utils.utils import get_minuto_atual
from utils.utils import get_segundo_atual

class Protocolo(models.Model):
    data = models.DateField(auto_now=True)
    hora = models.TimeField(auto_now=True)
    codigo = models.CharField(max_length=10)

    class Meta:
        db_table = 'protocolo'
        unique_together = ('data', 'hora', 'codigo')

    def __unicode__(self):
        final_codigo = re.split('\d{6}' ,self.codigo)[1]
        codigo_formatado = '%s/%s'%(re.split(final_codigo ,self.codigo)[0], final_codigo)

        return u'%s' % codigo_formatado


class ProtocoloArquivo(models.Model):
    protocolo = models.ForeignKey(
        Protocolo,
        blank=True,
        unique=True,
        null=True
    )
    ano = models.ForeignKey(Ano)
    mes = models.ForeignKey(Mes)
    municipio = models.ForeignKey(Municipio)
    cliente = models.ForeignKey(Cliente)
    proprietario = models.ForeignKey(
        User,
        blank=True,
        null=True,
        editable=False
    )
    observacao = models.TextField(blank=True, null=True)
    unique_together = ('ano', 'mes', 'municipio', 'cliente')

    class Meta:
        db_table = 'protocolo_arquivo'
        verbose_name = 'relatório de cliente'
        verbose_name_plural = 'relatórios de cliente'

    def __unicode__(self):
        return self.municipio.nome


class IndiceInformadoBase(models.Model):
    """ Classe base para informar índices.
    """
    valor = models.FloatField()


class TipoClienteIndiceInformadoManager(models.Manager):
    def get_query_set(self):
        qs = super(TipoClienteIndiceInformadoManager, self).get_query_set()
        return qs


class TipoClienteIndiceInformado(IndiceInformadoBase):
    indice = models.ForeignKey(IndiceTipoCliente)
    protocolo_arquivo = models.ForeignKey(ProtocoloArquivo)

    objects = TipoClienteIndiceInformadoManager()

    def save(self, *args, **kwargs):
        return super(TipoClienteIndiceInformado, self).save(*args, **kwargs)

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.indice.nome.capitalize()
        else:
            return self.indice.nome


class ClienteIndiceInformadoManager(models.Manager):
    def get_query_set(self):
        qs = super(ClienteIndiceInformadoManager, self).get_query_set()
        return qs


class ClienteIndiceInformado(IndiceInformadoBase):
    indice = models.ForeignKey(IndiceCliente)
    protocolo_arquivo = models.ForeignKey(ProtocoloArquivo)

    objects = ClienteIndiceInformadoManager()

    def save(self, *args, **kwargs):
        return super(ClienteIndiceInformado, self).save(*args, **kwargs)

    def __unicode__(self):
        if settings.CAPITALIZE:
            return self.indice.nome.capitalize()
        else:
            return self.indice.nome

class Arquivo(models.Model):
    """ O max_length do arquivo é a soma do max_length do
    estado, 100
    municipio, 100
    cliente, 200
    ano, 4
    mes + código_do_mes + _(underline), 20 + 3
    tipo_relatorio, 100
    extensão do arquivo '.pdf' e as barras separadoras. 3
    """
    tipo_relatorio = models.ForeignKey(TipoRelatorio)
    arquivo = models.FileField(upload_to='uploads', max_length=550)
    slug = models.SlugField(max_length=200, blank=True, unique=True, editable=False)
    protocolo_arquivo = models.ForeignKey(ProtocoloArquivo)

    class Meta:
        db_table = 'arquivo'

    def __unicode__(self):
        return self.arquivo.file.name.split('/')[len(self.arquivo.file.name.split('/'))-1]


def gera_versao(instance, salvar_em):
    """ Controla versão dos arquivos

    Verifica se o arquivo que está sendo enviado já existe, e em caso positivo
    renomeia o arquivo antigo antes de copiar o novo.

    O padrão do nome do arquivo é: nome_do_arquivoAAAAMMDDHHMMSS.pdf
    """
    try:
        if(Arquivo.objects.get(slug=instance.slug)):
            ano = unicode(get_ano_atual())
            mes = formata_dia_mes(unicode(get_mes_atual()))
            dia = formata_dia_mes(unicode(get_dia_atual()))
            hora = formata_dia_mes(unicode(get_hora_atual()))
            minuto = formata_dia_mes(unicode(get_minuto_atual()))
            segundo = formata_dia_mes(unicode(get_segundo_atual()))

            local_arquivo_original =\
                settings.MEDIA_ROOT + '/' + salvar_em + '/' +\
                instance.arquivo.field.get_filename(instance.arquivo.name)

            nome_arquivo_original = \
                re.split('\.',
                    instance.arquivo.field.get_filename(instance.arquivo.name)
                )

            NOME_ARQUIVO = 0
            EXTENSAO_ARQUIVO = 1

            destino = \
            settings.MEDIA_ROOT + '/' + salvar_em + '/' +\
                nome_arquivo_original[NOME_ARQUIVO] + \
                ano + mes + dia + hora + minuto + segundo + '.' +\
                nome_arquivo_original[EXTENSAO_ARQUIVO]

            # Remove todos os caracteres especiais.
            local_arquivo_original = normalize('NFKD', local_arquivo_original).\
                encode('ASCII', 'ignore')
            local_arquivo_original = local_arquivo_original.lower()

            # Remove todos os caracteres especiais.
            destino = normalize('NFKD', destino).encode('ASCII', 'ignore')
            destino = destino.lower()

            if os.path.isfile(local_arquivo_original):
                shutil.move(local_arquivo_original, destino)

    except(Arquivo.DoesNotExist):
        pass

# Manipuladores de sinais
def arquivo_pre_save(signal, instance, sender, **kwargs):
    """ Modifica o parâmetro upload_to do arquivo para o novo valor.

    Este método é invocado antes do método save() de controle.arquivos.Arquivo, 
    sua função é criar o slug para o arquivo com base nos parâmetros fornecidos 
    antes de enviar o arquivo.
    Também é sua função alterar o caminho onde será feito o upload do arquivo, 
    novamente usando os parâmetros fornecidos antes de enviar o arquivo.
    """
    if instance.DoesNotExist:
        if len(re.split('\/', instance.arquivo.field.upload_to)) == 1:
            if len(unicode(instance.protocolo_arquivo.mes.codigo)) == 1:
                mes = '0' + unicode(instance.protocolo_arquivo.mes.codigo)
            else:
                mes = unicode(instance.protocolo_arquivo.mes.codigo)

            mes = mes + '_' + instance.protocolo_arquivo.mes.nome

            novo_slug = '%s_%s_%s%s'%(\
                instance.tipo_relatorio.nome,\
                instance.protocolo_arquivo.cliente.nome,\
                mes,\
                instance.protocolo_arquivo.ano.ano)

            salvar_em = instance.arquivo.field.upload_to + '/' +\
                unicode(instance.protocolo_arquivo.municipio.estado.nome) + '/' +\
                unicode(instance.protocolo_arquivo.municipio.nome) + '/' +\
                unicode(instance.protocolo_arquivo.cliente.nome) + '/' +\
                unicode(instance.protocolo_arquivo.ano.ano) + '/' + mes

            # Remove todos os caracteres especiais.
            salvar_em = normalize('NFKD', salvar_em).encode('ASCII', 'ignore')
            salvar_em = salvar_em.lower()

            instance.arquivo.field.upload_to = salvar_em
            instance.arquivo.name = instance.tipo_relatorio.nome.lower() + '.pdf'
            instance.slug = slugify(novo_slug.lower())

            gera_versao(instance, salvar_em)


def arquivo_post_save(signal, instance, sender, **kwargs):
    """ Modifica o parâmetro upload_to do arquivo para o valor padrão.

    Este método é invocado após o método save() de controle.arquivos.Arquivo, 
    sua função é retornar a propriedade upload_to para o valor padrão.
    """

    old_upload_to = re.split('\/', instance.arquivo.field.upload_to, 1)[0]

    instance.arquivo.field.upload_to = old_upload_to


def protocolo_arquivo_pre_save(signal, instance, sender, **kwargs):
    if not instance.pk:
        protocolo = Protocolo.objects.get_or_create(codigo=gera_codigo_protocolo())[0]
        instance.protocolo = protocolo

def gera_codigo_protocolo():
    """ Gera o código do protocolo.

    O código gerado segue o seguinte formato: DDMMAACCCC onde:
    DD - Dia com dois dígitos
    MM - Mês com dois dígitos 
    AA - Ano com dois dígitos
    CCCC - Código sequencia por dia com quatro dígitos
    """
    dia = formata_dia_mes(unicode(datetime.date.today().day))
    mes = formata_dia_mes(unicode(datetime.date.today().month))
    ano = unicode(datetime.date.today().year).rsplit('20')[1]

    try:
        ultimo_protocolo = Protocolo.objects.latest('id')
        ultimo_codigo = re.split('\d{6}', ultimo_protocolo.codigo)[1]

        # O django por algum motivo está chamando esta função duas vezes o que
        # está fazendo com que sempre seja perdido um código de protocolo.
        # A condição abaixo impede que isso aconteça até que o problema seja
        #detectado
        if not ultimo_protocolo.protocoloarquivo_set.all():
            if ultimo_protocolo.data == datetime.date.today():
                ultimo_codigo = unicode(ultimo_codigo)
            else:
                ultimo_codigo = '1'
        else:
            if ultimo_protocolo.data == datetime.date.today():
                ultimo_codigo = unicode(int(ultimo_codigo) + 1)
            else:
                ultimo_codigo = '1'

    except(Protocolo.DoesNotExist):
        ultimo_codigo = '1'

    if len(ultimo_codigo) == 1:
        ultimo_codigo = '000' + ultimo_codigo
    elif len(ultimo_codigo) == 2:
        ultimo_codigo = '00' + ultimo_codigo
    elif len(ultimo_codigo) == 3:
        ultimo_codigo = '0' + ultimo_codigo

    codigo = '%s%s%s%s' % (dia, mes, ano, ultimo_codigo)

    return codigo


signals.pre_save.connect(protocolo_arquivo_pre_save, sender=ProtocoloArquivo)
signals.pre_save.connect(arquivo_pre_save, sender=Arquivo)
signals.post_save.connect(arquivo_post_save, sender=Arquivo)
